unit avltree;

interface

uses
    SysUtils, Classes, RTLConsts;

type
  TBalance = (brLeft, brEqual, brRight);
  TDumpNodes = (dnPreorder, dnInorder, dnPostorder);
  TAVLNode = class;

  TAVLNode = class(TObject)
 	private
    FItem : Pointer;
  public
    ID : Integer;
    LeftChild, RightChild : TAVLNode;
    Balance               : TBalance;
    property Item : Pointer read FItem write FItem;
    constructor Create;
    destructor Destroy; override;
  end;

  TAVLTree = class(TObject)
  private
    FCount : integer;
    FRoot  : TAVLNode;
  protected
    procedure  AddNode(var parent : TAVLNode; newID : Integer; value : Pointer; var grow : Boolean);
    procedure  AdjustLeftGrow(var parent : TAVLNode);
    procedure  AdjustRightGrow(var parent : TAVLNode);
    function   Get(Index: Integer): Pointer;
    function   InternalGet(var parent: TAVLNode; index : integer; var bolFind:Boolean):pointer;
    procedure  InternalDumpNodes(var parent : TAVLNode; var s : string; search_algo:TDumpNodes);
  public
    procedure  Add(newID : Integer; value : Pointer);
    function   DumpNodes(search_algo:TDumpNodes):String;
    property   Count : Integer read FCount;
    property   Items[Index: Integer]: Pointer read Get; default;
    destructor Destroy; override;
  end;

implementation


{ TAVLNode }

constructor TAVLNode.Create;
begin
  //
end;

destructor TAVLNode.Destroy;
begin
  if (LeftChild <> nil) then LeftChild.Free;
  if (RightChild <> nil) then RightChild.Free;

  inherited;
end;

{ TAVLTree }

//c[ɒlǉ鏈
procedure TAVLTree.Add(newID: Integer; value: Pointer);
var
  grow : Boolean;
begin
  //Add\bh́A܂RootlǉT
  grow := False;
  AddNode(FRoot, newID, value, grow);
end;

//m[h̒ǉċAIɍs
procedure TAVLTree.AddNode(
  var parent: TAVLNode;
  newID: Integer;
  value : Pointer;
  var grow: Boolean
  );
begin
  //؂̍Ő[܂ō~肽ꍇAɃm[hǉ
  if (parent = nil) then
  begin
    parent := TAVLNode.Create;
    parent.ID := newID;
    parent.Balance := brEqual;
    parent.Item := value;
    grow := True;
    Inc(FCount);
    exit;
  end;

  //newID݂̐ߓ_ID菬̏
  //ɉĂ
  if (newID < parent.ID) then
  begin
    //֐ߓ_ǉ
    AddNode(parent.LeftChild, newID, value, grow);

    //؂ςꍇAgrow TrueŕԂĂ
    //FalsȅꍇAoX͕sv
    if (grow = False) then exit;

    if (parent.Balance = brRight) then
    begin
      //X͉E̍傫ꍇ
      //ɐVߓ_ǉꂽ̂ŁAŃoX
      parent.Balance := brEqual;

      //̃m[hɂ́A[xωĂȂƒʒm
      grow := False;
    end else if (parent.Balance = brEqual) then
    begin
      //XoXĂ̂ŁAɐߓ_ǉꂽ
      //[ԂɂȂ
      parent.Balance := brLeft;
    end else
    begin
      //X̍傫̂ŁA
      //ɐߓ_ǉꂽ߁AoXKvƂȂ
      AdjustLeftGrow(parent);
      grow := False;
    end;

  end else
  //newID݂̐ߓ_ID傫ꍇ̏
  //EɉĂ
  if (newID > parent.ID) then
  begin
    //Eɐߓ_ǉ
    AddNode(parent.RightChild, newID, value,  grow);

    //؂ςꍇAgrow TrueŕԂĂ
    //FalsȅꍇAoX͕sv
    if (grow = False) then exit;

    if (parent.Balance = brLeft) then
    begin
      //X͍̍傫ꍇ
      //EɐVߓ_ǉꂽ̂ŁAŃoX@
      parent.Balance := brEqual;
      grow := False;
    end else
    if (parent.Balance = brEqual) then
    begin
      //XoXĂ̂ŁAEɐߓ_ǉꂽ
      //E[ԂɂȂ
      parent.Balance := brRight;
    end else
    begin
      //XE̍傫̂
      //Eɐߓ_ǉꂽ߁AoXKvɂȂ
      AdjustRightGrow(parent);
      grow := False;
    end;
  end else
  begin
    //newIDƌ݂̐ߓ_IDꍇ́Am[h̒l
    parent.Item := value;
    grow := False;
  end;
end;

//c[̍ŃoXꂽƂ̏
procedure TAVLTree.AdjustLeftGrow(var parent: TAVLNode);
var
  OrgLeftChild, OrgGrandChild : TAVLNode;
begin
  OrgLeftChild := parent.LeftChild;
  if (OrgLeftChild.Balance = brLeft) then
    begin
      //̍ŃoXꂽ̂ŁAE]
      parent.LeftChild := OrgLeftChild.RightChild;
      OrgLeftChild.RightChild := parent;
      parent.Balance := brEqual;
      parent := OrgLeftChild;
    end else
    begin
      //̉EŃoXꂽ̂ŁA|E]
      OrgGrandchild := OrgLeftchild.RightChild;
      OrgLeftchild.RightChild := OrgGrandChild.LeftChild;
      OrgGrandchild.LeftChild := OrgLeftchild;
      parent.LeftChild := OrgGrandChild.RightChild;
      OrgGrandChild.RightChild := parent;
      if (OrgGrandChild.Balance = brLeft) then
        parent.Balance := brRight
      else
        parent.Balance := brEqual;
      if (OrgGrandchild.Balance = brRight) then
        OrgLeftchild.Balance := brLeft
       else
        OrgLeftchild.Balance := brEqual;
       parent := OrgGrandChild;
    end;
    parent.Balance := brEqual;
end;

//c[̉EŃoXꂽƂ̏
procedure TAVLTree.AdjustRightGrow(var parent: TAVLNode);
var
  OrgRightChild, OrgGrandChild : TAVLNode;
begin
  OrgRightChild := parent.RightChild;
  if (OrgRightChild.Balance = brRight) then
    begin
      //ẺEŃoXꂽ̂ŁA]
      parent.RightChild := OrgRightChild.LeftChild;
      OrgRightChild.LeftChild := parent;
      parent.Balance := brEqual;
      parent := OrgRightChild;
    end else
    begin
      //E̍ŃoXꂽ̂ŁAE|]
      OrgGrandchild := OrgRightchild.LeftChild;
      OrgRightchild.LeftChild := OrgGrandChild.RightChild;
      OrgGrandChild.RightChild := OrgRightChild;
      parent.RightChild := OrgGrandChild.LeftChild;
      OrgGrandChild.LeftChild := parent;
      if (OrgGrandChild.Balance = brRight) then
        parent.Balance := brLeft
      else
        parent.Balance := brEqual;
      if (OrgGrandchild.Balance = brLeft) then
        OrgRightChild.Balance := brRight
       else
        OrgRightChild.Balance := brEqual;
       parent := OrgGrandChild;
    end;
    parent.Balance := brEqual;
end;

destructor TAVLTree.Destroy;
begin
  if (FRoot <> nil) then FRoot.Free;

  inherited;
end;

//c[̓IDoāAŕԂ
function TAVLTree.DumpNodes(search_algo:TDumpNodes): String;
begin
  InternalDumpNodes(FRoot, result, search_algo);
end;

//c[̃CfbNXQƂɂl̎擾
function TAVLTree.Get(Index: Integer): Pointer;
var
  bolFind:Boolean;
begin
  Result := InternalGet(FRoot, Index, bolFind);
  if (bolFind = False) then raise EListError.Createfmt(LoadResString(@SListIndexError), [Index]);
end;

//c[̓Ԃ_v鏈
procedure TAVLTree.InternalDumpNodes(var parent: TAVLNode; var s: string; search_algo:TDumpNodes);
  procedure make_result;
  begin
    if (s <> '') then s := s + ', ';
    s := s + 'ID=' + IntToStr(parent.ID);
  end;
begin
  //s͂ŏ
  if (search_algo = dnPreorder) then make_result;

  if (parent.LeftChild <> nil) then InternalDumpNodes(parent.LeftChild, s, search_algo);

  //ʂ肪͂ŏ
  if (search_algo = dnInorder) then make_result;

  if (parent.RightChild <> nil) then InternalDumpNodes(parent.RightChild, s, search_algo);

  //A肪͂ŏ
  if (search_algo = dnPostorder) then make_result;
end;

//c[̃f[^̎擾ċAIɍs
function TAVLTree.InternalGet(var parent: TAVLNode; index : integer; var bolFind:Boolean): pointer;
var
  tmp:Pointer;
begin
  if (parent.ID = Index) then
  begin
    result := parent.item;
    bolFind := True;
    exit;
  end;
  if (parent.LeftChild <> nil) then
  begin
    tmp := InternalGet(parent.LeftChild, index, bolFind);
    if (bolFind = True) then
    begin
      result := tmp;
      exit;
    end;
  end;
  if (parent.RightChild <> nil) then
  begin
    tmp := InternalGet(parent.RightChild, index, bolFind);
    if (bolFind = True) then
    begin
      result := tmp;
      exit;
    end;
  end;
end;

end.

